import { Checkbox, SelectProps, Tag } from 'ant-design-vue'
import { BasicColumn, FormProps } from '@/components/Table'

/**
 * 支持的数据源类型
 */
export const databaseTypeOptions: SelectProps['options'] = [
    {
        label: '常规表',
        value: 'General',
        options: [
            {
                label: '常规单表',
                value: 'SingleGeneral',
            },
            {
                label: '常规主表',
                value: 'MasterGeneral',
                disabled: true,
            },
            {
                label: '常规从表',
                value: 'SlaveGeneral',
                disabled: true,
            },
        ],
    },
    {
        label: '时序表',
        value: 'TimeSeries',
        options: [
            {
                label: '时序单表',
                value: 'SingleTimeSeries',
                disabled: true,
            },
            {
                label: '业务时序表',
                value: 'BusinessTimeSeries',
                disabled: true,
            },
        ],
    },
]

// ================================
export function getColumns(): BasicColumn[] {
    return [
        {
            title: '表类型',
            dataIndex: 'type',
        },
        {
            title: '表名',
            dataIndex: 'name',
        },
        {
            title: '表描述',
            dataIndex: 'describe',
        },
        {
            title: '版本',
            dataIndex: 'version',
            sorter: true,
        },
        {
            title: '同步状态',
            dataIndex: 'sync',
            sorter: true,
            customRender: ({ value }) => <Tag color={value ? 'green' : 'red'}>{value ? '已同步' : '未同步'}</Tag>,
        },
        {
            title: '创建时间',
            dataIndex: 'createTime',
            sorter: true,
            width: 180,
        },
    ]
}
export function getDataSource() {
    return (() => {
        const data = []

        data.push({
            id: '1',
            type: '单表',
            name: 'bs_staff',
            describe: '人员信息表',
            version: '1',
            sync: true,
            createTime: '2023-02-26 15:36:01',
        })

        return data
    })()
}
export function getFormConfig(): Partial<FormProps> {
    return {
        layout: 'inline',
        name: 'query',
        colon: true,
        compact: true,
        schemas: [
            {
                field: `type`,
                label: `表类型`,
                component: 'Select',
                componentProps: {
                    options: databaseTypeOptions,
                    mode: 'multiple',
                    style: {
                        width: '180px',
                    },
                },
            },
            {
                field: `name`,
                label: `表名`,
                component: 'Input',
            },
            {
                field: `describe`,
                label: `表描述`,
                component: 'Input',
            },
        ],
        showAdvancedButton: false,
        submitButtonOptions: {
            preIcon: 'material-symbols:search',
        },
    }
}

// ================================
export type FieldType =
    | 'Integer'
    | 'Float'
    | 'Double'
    | 'BigDecimal'
    | 'Char'
    | 'String'
    | 'Text'
    | 'Boolean'
    | 'LocalDate'
    | 'LocalTime'
    | 'LocalDateTime'
    | 'Point'
    | 'Line'
    | 'Polygon'
    | 'Geometry'
    | 'Blob'
    | 'JSONObject'
    | 'JSONArray'
/**
 * 支持的数据源类型
 */
export const fieldTypeOptions: SelectProps['options'] = [
    {
        label: '数值类型',
        value: 'Number',
        options: [
            {
                label: '整数',
                value: 'Integer',
            },
            {
                label: '单精度小数',
                value: 'Float',
            },
            {
                label: '双精度小数',
                value: 'Double',
            },
            {
                label: '高精度小数',
                value: 'BigDecimal',
            },
        ],
    },
    {
        label: '字符串类型',
        value: 'Varchar',
        options: [
            {
                label: '字符',
                value: 'Char',
            },
            {
                label: '字符串',
                value: 'String',
            },
            {
                label: '大文本',
                value: 'Text',
            },
        ],
    },
    {
        label: '布尔类型',
        value: 'Bool',
        options: [
            {
                label: '布尔',
                value: 'Boolean',
            },
        ],
    },
    {
        label: '日期类型',
        value: 'Date',
        options: [
            {
                label: '日期',
                value: 'LocalDate',
            },
            {
                label: '时间',
                value: 'LocalTime',
            },
            {
                label: '日期时间',
                value: 'LocalDateTime',
            },
        ],
    },
    {
        label: '空间类型',
        value: 'GIS',
        options: [
            {
                label: '坐标点',
                value: 'Point',
            },
            {
                label: '空间线',
                value: 'Line',
            },
            {
                label: '空间面',
                value: 'Polygon',
            },
            {
                label: '空间几何',
                value: 'Geometry',
            },
        ],
    },
    {
        label: '其他',
        value: 'Other',
        options: [
            {
                label: 'Blob 对象',
                value: 'Blob',
            },
            {
                label: 'JSON 对象',
                value: 'JSONObject',
            },
            {
                label: 'JSON 数组',
                value: 'JSONArray',
            },
        ],
    },
]
const fieldTypeMap: Recordable = fieldTypeOptions
    .map((item) => item.options)
    .flat()
    .reduce((all, item) => {
        all[item.value] = item.label
        return all
    }, {})

export interface DataSourceSchema {
    field?: string
    comment?: string
    type?: FieldType
    len?: number
    decimal?: number
    pk?: boolean
    empty?: boolean
    transient?: boolean
    editable?: boolean
    disabled?: boolean
    [key: string]: any
}

/**
 * 获取数据源字段列表
 * @returns Column[]
 */
export function getDataSourceSchemaColumns(): BasicColumn[] {
    return [
        {
            title: '字段名称',
            dataIndex: 'field',
            editRow: true,
            editRule: true,
            editComponent: 'Input',
            editDynamicDisabled: ({ record }: { record: DataSourceSchema }) => record.disabled || record.disableField,
        },
        {
            title: '字段备注',
            dataIndex: 'comment',
            editRow: true,
            editRule: true,
            editComponent: 'Input',
            editDynamicDisabled: ({ record }: { record: DataSourceSchema }) => record.disabled || record.disableComment,
        },
        {
            title: '字段类型',
            dataIndex: 'type',
            editRow: true,
            editComponent: 'Select',
            editComponentProps: {
                options: fieldTypeOptions,
            },
            editDynamicDisabled: ({ record }) => record.disabled || record.disableType,
            editRender: ({ record }: { record: DataSourceSchema }) => fieldTypeMap[record.type],
        },
        {
            title: '字段长度',
            dataIndex: 'len',
            editRow: true,
            editComponent: 'InputNumber',
            editDynamicDisabled: ({ record }: { record: DataSourceSchema }) => record.disabled || record.disableLen,
        },
        {
            title: '小数点',
            dataIndex: 'decimal',
            editRow: true,
            editComponent: 'InputNumber',
            editDynamicDisabled: ({ record }: { record: DataSourceSchema }) => record.disabled || record.disableDecimal,
        },
        {
            title: '是否主键',
            dataIndex: 'pk',
            customRender: ({ record }: { record: DataSourceSchema }) => (
                <Checkbox
                    v-model:checked={record.pk}
                    disabled={record.disabled || record.disablePK || !record.editable}
                />
            ),
        },
        {
            title: '允许空值',
            dataIndex: 'empty',
            customRender: ({ record }: { record: DataSourceSchema }) => (
                <Checkbox
                    v-model:checked={record.empty}
                    disabled={record.disabled || record.disableEmpty || !record.editable}
                />
            ),
        },
        {
            title: '是否同步数据库',
            dataIndex: 'transient',
            customRender: ({ record }: { record: DataSourceSchema }) => (
                <Checkbox
                    v-model:checked={record.transient}
                    disabled={record.disabled || record.disableTransient || !record.editable}
                />
            ),
        },
    ]
}

export function getDefaultSchemaFieldDataSourcesByType(type: string): DataSourceSchema[] {
    if (type === 'SingleGeneral') {
        return [
            {
                id: 'id',
                field: 'id',
                comment: '主键',
                type: 'String',
                len: 32,
                pk: true,
                empty: false,
                transient: true,
                disabled: true,
            },
            {
                id: 'createBy',
                field: 'createBy',
                comment: '创建人',
                type: 'String',
                len: 32,
                pk: false,
                empty: false,
                transient: true,
                disabled: true,
            },
            {
                id: 'createTime',
                field: 'createTime',
                comment: '创建时间',
                type: 'LocalDateTime',
                len: 0,
                decimal: 0,
                pk: false,
                empty: false,
                transient: true,
                disabled: true,
            },
            {
                id: 'updateBy',
                field: 'updateBy',
                comment: '修改人',
                type: 'String',
                len: 32,
                pk: false,
                empty: false,
                transient: true,
                disabled: true,
            },
            {
                id: 'updateTime',
                field: 'updateTime',
                comment: '修改时间',
                type: 'LocalDateTime',
                len: 0,
                decimal: 0,
                pk: false,
                empty: false,
                transient: true,
                disabled: true,
            },
        ]
    }

    return []
}
